@@ -1,8 +1,8 @@
 /* ------------------------------------------------------------------------- */
 /*   "syntax" : Syntax analyser and compiler                                 */
 /*                                                                           */
-/*   Part of Inform 6.34                                                     */
-/*   copyright (c) Graham Nelson 1993 - 2017                                 */
+/*   Part of Inform 6.35                                                     */
+/*   copyright (c) Graham Nelson 1993 - 2021                                 */
 /*                                                                           */
 /* ------------------------------------------------------------------------- */
 
@@ -71,13 +71,7 @@ extern void get_next_token_with_directives(void)
 
        This is called while parsing a long construct, such as Class or
        Object, where we want to support internal #ifdefs. (Although
-       function-parsing predates this and doesn't make use of it.)
-
-       (Technically this permits *any* #-directive, which means you
-       can define global variables or properties or what-have-you in
-       the middle of an object. You can do that in the middle of an
-       object, too. Don't. It's about as well-supported as Wile E.
-       Coyote one beat before the plummet-lines kick in.) */
+       function-parsing predates this and doesn't make use of it.) */
 
     int directives_save, segment_markers_save, statements_save;
 
@@ -143,6 +137,8 @@ extern int parse_directive(int internal_flag)
         TRUE if encountered with a # prefix inside a routine or object
         definition.
 
+        (Only directives like #ifdef are permitted inside a definition.)
+
         Returns: TRUE if program continues, FALSE if end of file reached.    */
 
     int routine_symbol, rep_symbol;
@@ -171,12 +167,15 @@ extern int parse_directive(int internal_flag)
         df_dont_note_global_symbols = TRUE;
         get_next_token();
         df_dont_note_global_symbols = FALSE;
-        if ((token_type != SYMBOL_TT)
-            || ((!(sflags[token_value] & UNKNOWN_SFLAG))
-                && (!(sflags[token_value] & REPLACE_SFLAG))))
+        if (token_type != SYMBOL_TT)
         {   ebf_error("routine name", token_text);
             return(FALSE);
         }
+        if ((!(sflags[token_value] & UNKNOWN_SFLAG))
+            && (!(sflags[token_value] & REPLACE_SFLAG)))
+        {   ebf_symbol_error("routine name", token_text, typename(stypes[token_value]), slines[token_value]);
+            return(FALSE);
+        }
 
         routine_symbol = token_value;
 
@@ -600,6 +599,11 @@ extern void parse_code_block(int break_label, int continue_label,
     {   do
         {   begin_syntax_line(TRUE);
             get_next_token();
+            
+            if ((token_type == SEP_TT) && (token_value == HASH_SEP))
+            {   parse_directive(TRUE);
+                continue;
+            }
             if (token_type == SEP_TT && token_value == CLOSE_BRACE_SEP)
             {   if (switch_clause_made && (!default_clause_made))
                     assemble_label_no(switch_label);
